@@ -3,7 +3,7 @@ from pathlib import Path
 import click
 
 from kks.ejudge_submit import submit_solution
-from kks.util.common import get_valid_session, load_links, prompt_choice, find_workspace
+from kks.util.common import get_valid_session, load_links, prompt_choice, find_problem_rootdir
 
 
 @click.command(short_help='Submit a solutions')
@@ -45,24 +45,13 @@ def submit(file, problem):
     click.secho(msg, fg=color)
 
 
-def find_problem_rootdir():
-    cwd = Path.cwd().resolve()
-    workspace = find_workspace(cwd)
-    if workspace is None:
-        return None
-    parts = cwd.relative_to(workspace).parts
-    if len(parts) < 2:
-        return None
-    return workspace / parts[0] / parts[1]
-
-
 def get_problem_id(rootdir):
     return '{}-{}'.format(*rootdir.parts[-2:])
 
 
 def find_solution():
     cwd = Path.cwd().resolve()
-    source_files = list(cwd.glob('*.c')) + list(cwd.glob('*.h'))
+    source_files = list(cwd.glob('*.[chSs]'))
     if len(source_files) == 0:
         click.secho('No source files found', fg='red', err=True)
         return None
