@@ -1,4 +1,5 @@
 from copy import copy
+from datetime import datetime, timedelta
 from itertools import groupby
 from urllib.parse import quote as urlquote
 
@@ -46,7 +47,7 @@ class Problem:
     def __init__(self, short_name, name, href, status, tests_passed, score):
         self.short_name = short_name
         self.name = name
-        self.href = href
+        self.href = href  # NOTE contains SID -> quickly becomes outdated
         self.status = status
         self.tests_passed = tests_passed
         self.score = score
@@ -135,6 +136,11 @@ class TaskScore:
     def bold(self):
         return self.status == Status.OK
 
+    def table_score(self):
+        if self.status == Status.TESTING and self.score is None:
+            return '??'
+        return self.score
+
 
 class Standings:
     def __init__(self, task_names, rows):
@@ -165,6 +171,15 @@ class StandingsRow:
         return self.is_self
 
 
+class ProblemInfo:
+    """Subset of task info table used for max score estimation"""
+
+    def __init__(self, full_score, run_penalty, current_penalty):
+        self.full_score = full_score
+        self.run_penalty = run_penalty
+        self.current_penalty = current_penalty
+
+
 class Statement:
 
     keep_info = ['Time limit:', 'Real time limit:', 'Memory limit:']
@@ -362,7 +377,7 @@ def extract_contest_name(task_name):
 
 def to_task_score(task_name, cell):
     score = cell.text
-    if score.isspace():
+    if not score or score.isspace():
         score = None
 
     status = Status.REVIEW if 'cell_attr_pr' in cell['class'] \
@@ -372,9 +387,6 @@ def to_task_score(task_name, cell):
         else Status.OK if score is not None \
         else Status.NOT_SUBMITTED
 
-    if status == Status.TESTING and score is None:
-        score = '??'
-
     return TaskScore(extract_contest_name(task_name), task_name, score, status)
 
 
@@ -417,6 +429,82 @@ def ejudge_report(link, session):
     return Report(comments, tests)
 
 
+def get_problem_info(problem, cache, session):
+    from bs4 import BeautifulSoup
+
+    # TODO
+    # - Should max_score be set to 0 after the hard deadline?
+    # - Can there be multiple soft deadlines?
+
+    # NOTE penalties are assumed to be the same for all tasks in a contest, it may be wrong
+
+    need_loading = False
+
+    full_scores = cache.get('full', {})
+    run_penalties = cache.get('run', {})
+
+    full_score = full_scores.get(problem.short_name)
+    run_penalty = run_penalties.get(problem.short_name)
+
+    penalty_key = f'p_{problem.contest}'
+    current_penalty = cache.get(penalty_key)
+
+    if full_score is None or run_penalty is None or current_penalty is None:  # new problem or need to update penalty
+        need_loading = True
+
+    if not need_loading:
+        return ProblemInfo(full_score, run_penalty, current_penalty)
+
+    def update_cache(full_score, run_penalty, current_penalty, soft_dl):
+        full_scores[problem.short_name] = full_score
+        run_penalties[problem.short_name] = run_penalty
+        cache.set('full', full_scores)
+        cache.set('run', run_penalties)
+
+        if current_penalty >= full_score and current_penalty != 0:
+            expiration = None
+            # max_score will not change
+            # NOTE may need patching for kr contests
+        else:
+            if soft_dl is not None:
+                expiration = soft_dl
+            else:
+                expiration = timedelta(days=4, hours=23, minutes=55)
+        cache.set(penalty_key, current_penalty, expiration)
+
+        return ProblemInfo(full_score, run_penalty, current_penalty)
+
+    page = session.get(problem.href)
+    soup = BeautifulSoup(page.content, 'html.parser')
+    task_area = soup.find('div', {'id': 'probNavTaskArea'})
+    problem_info = task_area.find('table', {'class': 'line-table-wb'})
+
+    full_score = 0
+    run_penalty = 0  # may be incorrect for kr
+    current_penalty = 0
+    soft_dl = None
+
+    if problem_info is None:
+        # may happen in kr contests?
+        return update_cache(full_score, run_penalty, current_penalty, None)
+
+    # TODO "Full score" can be missing (sm01-3)
+    # in this case we should use max score from the table as full
+    # If the penalty can be missing too, we should also parse hard deadlines
+    for row in problem_info.find_all('tr'):
+        key, value = row.find_all('td')
+        if key.text == 'Full score:':
+            full_score = int(value.text)
+        elif key.text == 'Run penalty:':
+            run_penalty = int(value.text)
+        elif key.text == 'Current penalty:':
+            current_penalty = int(value.text)
+        elif key.text == 'Next soft deadline:':
+            soft_dl = datetime.strptime(value.text, '%Y/%m/%d %H:%M:%S')
+
+    return update_cache(full_score, run_penalty, current_penalty, soft_dl)
+
+
 def check_session(links, session):
     summary = links.get(LinkTypes.SUMMARY, None)
     if summary is None:
